package agent;

/** The representation of the agent. This is used to keep external state
    information about the agent and serves as a container class for
    the actual agent. Typically this should be subclasses so that
    it can contain environment specific information. */
public class AgentRepresentation {

  protected Agent agent;

  public AgentRepresentation(Agent a) {
    agent = a;
  }

  public Agent getAgent() {
    return agent;
  }

  /** Provide a Percept to the agent. This function is called by the
      environment at the beginning of each of the agent's turn.
      If the agent has internal state, this method should
      also update it. */
  public void see(Percept p) {
    agent.see(p);
  }

  /** Have the agent select its next action to perform. Implements the
      action: Per -> Ac function or the action: I -> Ac function,
      depending on whether or not the agent has internal state. */
  public Action selectAction() {
    return agent.selectAction();
  }

}
