package pacworld;

import agent.AgentRepresentation;

/** The representation of a Package World agent. Include information about
    the agent's id, position, and whether or not it is holding anything. 
    Also serves as a proxy for providing percepts to receiving actions from
    the actual agent. */
public class PacAgentRep extends AgentRepresentation {

  int id;
  int x;
  int y;

  /** The package currently held by the agent, or null if the agent is
      not holding a package. */
  Package heldPackage;

  /** Did the agent (or the package it was holding) bump into something on
      its last move. */
  boolean bumped;

  public PacAgentRep(int id, PacAgent a) {
    super(a);
    this.id = id;
    x = 0;
    y = 0;
    heldPackage = null;
    bumped = false;
  }

  public boolean getBumped() {
    return bumped;
  }

  public int getId() {
    return id;
  }

  public Package getHeldPackage() {
    return heldPackage;
  }

  public int getX() {
    return x;
  }

  public int getY() {
    return y;
  }

  public boolean hasBumped() {
    return bumped;
  }

  public boolean isHolding() {
    if (heldPackage == null)
      return false;
    else
      return true;
  }

  public void setBumped(boolean bumped) {
    this.bumped = bumped;
  }

  public void setHeldPackage(Package p) {
    heldPackage = p;
  }

  public void setX(int x) {
    this.x = x;
  }

  public void setY(int y) {
    this.y = y;
  }

}
