package pacworld;

import agent.Percept;
import agent.Agent;
import java.util.Vector;
import java.awt.Point;

/** A percept in the package delivery world. */
public class PacPercept extends Percept{

  protected boolean bump;
  protected VisibleAgent[] visAgents;
  protected VisiblePackage[] visPackages;
  protected String[] messages;
  protected int worldSize;


  /** Construct a package delivery world percept. The agents can
      see everything in the world. If the agent moved into an
      obstacle on the previous turn the agent will feel a bump. */
  public PacPercept(PackageState state, PacAgentRep agent) {

    super(state,agent);

    // consider eventually giving the agent a limited view of the world

    worldSize = state.getMapSize();

    PacAgentRep[] agentReps = state.getAgentReps();
    int numAgents = agentReps.length;
    visAgents = new VisibleAgent[numAgents];
    for (int i=0; i < numAgents; i++) {
      PacAgentRep arep = agentReps[i];
      VisibleAgent vagent = new VisibleAgent(i, arep.getX(), arep.getY());
      visAgents[i] = vagent;
    }
      
    Vector packages = state.getPackages();
    visPackages = new VisiblePackage[state.getNumPackages()];
    int vi = 0;
    for (int i=0; i < state.getOrigNumPackages(); i++) {
      if (packages.elementAt(i) != null) {
	Package pack = (Package)(packages.elementAt(i));
	VisiblePackage vpack;
	vpack = new VisiblePackage(pack);
	visPackages[vi] = vpack;
	vi++;
      }
    }
    messages = (String[])(state.getMessages().clone());

    // determine bump
    if (state.bumped(agent))
      bump = true;
    else
      bump = false;

    // printPercept();
  }

  /** Returns true if the percept reflects that the agent bumped into
      an obstacle as a result of its most recent action. */
  public boolean feelBump() {
    return bump;
  }


  /** Returns the length of the world. Note, the world is a square,
      so this also doubles as its height. */
  public int getWorldSize() {
    return worldSize;
  }
      
  /** Returns an array of the visible agents. Each element is of type
      VisibleAgent. */
  public VisibleAgent[] getVisAgents() {
    return visAgents;
  }

  /** Return an array of the visible packages. Each element is of type
      VisiblePackage. Only packages that have not been delivered will
      appear. */
  public VisiblePackage[] getVisPackages() {
    return visPackages;
  }

  /** Returns an array of strings representing messages sent since the
      agent's last turn. The array has as many elements as there are
      agents, and if any agent didn't say anything on it's prior turn
      then the corresponding element will be empty. */
  public String[] getMessages() {
    return messages;
  }


  public void printPercept() {
    System.out.println();
    System.out.println("Percept:");
    for (int i=0; i < visAgents.length; i++)
      System.out.println(visAgents[i]);
    for (int i=0; i < visPackages.length; i++)
      System.out.println(visPackages[i]);
    for (int i=0; i < messages.length; i++)
      System.out.println(messages[i]);
    System.out.println();
    System.out.println();
  }

}
